@@ -17,9 +17,19 @@
 
 source "$FEISTY_MEOW_SCRIPTS/core/launch_feisty_meow.sh"
 
+# define some variables to avoid going nuts seeing 0 and 1 everywhere.
+RET_OKAY=0
+RET_FAIL=1
+
+# prints out a message with a time-stamp prefixed to it.
+function log_it()
+{
+  echo -e "$(date_stringer): $@"
+}
+
 if [ ! -z "$CLEAN" ]; then
-  echo "in cleaning mode, will not build dependencies."
-  exit 0
+  log_it "in cleaning mode, will not build dependencies."
+  exit $RET_OKAY
 fi
 
 # uncomment to enable debugging noises.
@@ -29,14 +39,14 @@ fi
 # information, rather than trying to juggle positional parameters everywhere.
 
 # the list of dependencies being accumulated.
-declare -a dependency_accumulator=()
+declare -A dependency_accumulator
 
 # a set of files that are known to be bad, since we cannot find them.
-declare -a bad_files=()
+declare -A bad_files
 
 # makes sure we don't keep looking at files even when they're neither
 # bad nor listed as dependencies.
-declare -a boring_files=()
+declare -A boring_files
 
 # this directory is not allowed to participate in the scavenging
 # because it's where the tool was pointed at.  if we allowed files in
@@ -54,205 +64,207 @@ TAB_CHAR="$(echo -e -n '\t')"
 # a zero success value is returned if the file has been seen before,
 # and a non-zero failure value for when the file is totally new.
 function seen_already {
-  if existing_dep "$1"; then return 0; fi  # added it to list already.
-  if bad_file "$1"; then return 0; fi  # known to suck.
-  if boring_file "$1"; then return 0; fi  # we already saw it.
-  return 1  # we had not seen this one, so we return an error.
+  if existing_dep "$1"; then return $RET_OKAY; fi  # added it to list already.
+  if bad_file "$1"; then return $RET_OKAY; fi  # known to suck.
+  if boring_file "$1"; then return $RET_OKAY; fi  # we already saw it.
+  return $RET_FAIL  # we had not seen this one, so we return an error.
 }
 
 # adds a new dependency at the end of the list.
 function add_new_dep {
   # make sure we haven't already processed this.
-  local dep="$1"
+  local dep="$1"; shift
   if seen_already "$dep"; then
     if [ ! -z "$DEBUG_BUILDOR_GEN_DEPS" ]; then
-      echo "bailing since seen: $dep"
+      log_it "bailing since seen: $dep"
     fi
-    return 1
+    return $RET_FAIL
   fi
   if [ ! -z "$DEBUG_BUILDOR_GEN_DEPS" ]; then
-    echo "had not seen before: $dep"
+    log_it "had not seen before: $dep"
   fi
 
   if [ ! -z "$DEBUG_BUILDOR_GEN_DEPS" ]; then
-    echo "new dependency: $dep"
+    log_it "new dependency: $dep"
   fi
 
-  dependency_accumulator+=($dep)
-  return 0
+  dependency_accumulator[$dep]=yo
+  return $RET_OKAY
 }
 
 # checks the existing dependencies to see if the first parameter is already
 # listed.  if this is the case, zero is returned (meaning success).  if
 # the dependency is missing, then -1 is return to indicate an error.
 function existing_dep {
-#hmmm: below is not very efficient!
-  for currite in ${dependency_accumulator[*]}; do
-    if [ "$currite" == "$1" ]; then return 0; fi
-  done
-  return 1
+  if [ ! -z "${dependency_accumulator[$1]}" ]; then
+    return $RET_OKAY
+  fi
+  return $RET_FAIL
 }
 
 # reports whether a file name has already been processed.
 function boring_file {
-
-#hmmm: below might not be very efficient!
-  for currite in ${boring_files[*]}; do
-    if [ "$currite" == "$1" ]; then return 0; fi
-  done
-  return 1
+  if [ ! -z "${boring_files[$1]}" ]; then
+    return $RET_OKAY
+  fi
+  return $RET_FAIL
 }
 
 # reports whether a file name has already been found to be missing.
 function bad_file {
-
-#hmmm: below also is not very efficient!
-  for currite in ${bad_files[*]}; do
-    if [ "$currite" == "$1" ]; then return 0; fi
-  done
-  return 1
-}
-
-# checks whether an item is already contained in a list.  the first parameter
-# is taken as the item that one wants to add.  the second through n-th
-# parameters are taken as the candidate list.  if the item is present, then
-# zero is returned to indicate success.  otherwise a non-zero return value
-# indicates that the item was not yet present.
-function already_listed {
-  to_find=$1
-  shift
-  while (( $# > 0 )); do
-    # return that we found it if the current item matches.
-    if [ "$to_find" == "$1" ]; then return 0; fi
-    shift  # toss next one out.
-  done
-  # failed to match it.
-  return 1
-}
-
-# finds the index of a particular element in the remainder of a list.
-# the variable __finders_indy will be set to -1 for no match, or it will be the
-# index of the element if the item was found.
-__finders_indy=-1
-function find_in_array {
-  local to_find=$1
-#echo find_in_array needs: $to_find
-  shift
-#echo restargs finder: $*
-  local indy=0
-  while (( $# > 0 )); do
-    # return that we found it if the current item matches.
-#echo "find_in_array posn $indy has $1"
-    if [ "$to_find" == "$1" ]; then
-#echo "FOUND $to_find at $indy"
-       __finders_indy=$indy
-       return 0
-    fi
-    shift  # toss next one out.
-    indy=$(expr $indy + 1)
-#echo "find_in_array indy now $indy "
-  done
-  _finders_indy=-1
-  # failed to match it.
-  return 1
+  if [ ! -z "${bad_files[$1]}" ]; then
+    return $RET_OKAY
+  fi
+  return $RET_FAIL
 }
 
 ############################################################################
 #
 # this variable gets stored into when resolve_filename runs.
-declare -a resolve_target_array=()
+declare -A resolve_target_array
 #
 # this variable is used internally by resolve_filename.  it should not need
 # to be reset between runs on different files because the source hierarchy
 # is not supposed to be getting files deleted or added while the deps are
 # being geneated.
-declare -a resolve_matches_src=()
-declare -a resolve_matches_dest=()
+declare -A resolve_matches
 #
 # tries to find a filename in the library hierarchy.
-function resolve_filename {
+function resolve_filename() {
+  # wipe out prior global contents from resolved paths.
+  unset resolve_target_array
+  declare -gA resolve_target_array
+
   local code_file=$1
   if [ ! -z "$DEBUG_BUILDOR_GEN_DEPS" ]; then
-    echo "resolving: $code_file"
+    log_it "resolving: $code_file"
   fi
   if [ -f "$code_file" ]; then
     # that was pretty easy.
-    resolve_target_array=($code_file)
-    return 0
+    resolve_target_array[$code_file]=zuh
+    return $RET_OKAY
   fi
   if [ ! -z "$DEBUG_BUILDOR_GEN_DEPS" ]; then
-    echo "MUST seek: $code_file"
+    log_it "MUST seek: $code_file"
   fi
 
   local dir=$(dirname "$code_file")
+  if [ "$dir" == "." ]; then
+    # we plan to behave as if there was no directory for this case.
+    # we can't include a simple dot in our searches below.
+    unset dir
+  fi
   local base=$(basename "$code_file")
   local src_key="$dir/$base"
+  if [ -z "$dir" ]; then
+    # with no directory, or current directory, this just has to be keyed by the basename.
+    src_key="$base"
+  fi
   if [ ! -z "$DEBUG_BUILDOR_GEN_DEPS" ]; then
-    echo "src_key: $src_key"
+    log_it "src_key: $src_key"
   fi
 
   # see if we can find that element in the previously resolved items.
-  if find_in_array "$src_key" ${resolve_matches_src[*]}; then
-    local found_indy=$__finders_indy
-    resolve_target_array=(${resolve_matches_dest[$found_indy]})
+  if [ ! -z "${resolve_matches[$src_key]}" ]; then
+    local flounder="${resolve_matches[$src_key]}"
+    resolve_target_array[$flounder]=blup
     if [ ! -z "$DEBUG_BUILDOR_GEN_DEPS" ]; then
-      echo "FOUND \"$src_key\" AT ${resolve_matches_dest[$found_indy]}"
+      log_it "FOUND \"$src_key\" AT $flounder"
     fi
-    return 0
+    return $RET_OKAY
   fi
 
-  # reset our global list.
-  resolve_target_array=()
   if [ ! -z "$DEBUG_BUILDOR_GEN_DEPS" ]; then
-    echo "HAVING TO FIND: $dir and $base"
+    log_it "HAVING TO FIND: dir='$dir' and base='$base'"
   fi
   if [ -z "$dir" ]; then
-    resolve_target_array=($(find "$BUILD_TOP" -iname "$base"))
+    local -a init_list="$(find "$BUILD_TOP" -iname "$base" | sed -e "s%\\(.*\\)%[\1]=lop %")"
+#log_it "no dir case, init list is: ${init_list}"
+    if [ ${#init_list} -gt 0 ]; then
+      eval resolve_target_array=(${init_list})
+#log_it "after no dir case addition, resolve targets are: " ${!resolve_target_array[@]}
+    fi
   else
-    resolve_target_array=($(find "$BUILD_TOP" -iname "$base" | grep "$dir.$base"))
+    # a little tricky here, since we can't emit the associative array init line too early, or it will match for the grep.
+    local -a init_list="$(find "$BUILD_TOP" -iname "$base" | grep "$dir.$base" | sed -e "s%\\(.*\\)%[\1]=oof %")"
+#log_it "with dir case, init list is: ${init_list}"
+    if [ ${#init_list} -gt 0 ]; then
+      eval resolve_target_array=(${init_list})
+#log_it "after dir case addition, resolve targets are: " ${!resolve_target_array[@]}
+    fi
   fi
   if [ ! -z "$DEBUG_BUILDOR_GEN_DEPS" ]; then
-    echo resolved to: ${resolve_target_array[*]}
-    echo size of resolve array=${#resolve_target_array[*]}
+    log_it "resolved to: ${!resolve_target_array[@]}"
+    log_it "size of resolve array=${#resolve_target_array[@]}"
   fi
-  if [ ${#resolve_target_array[*]} -eq 1 ]; then
+  if [ ${#resolve_target_array[@]} -eq 1 ]; then 
+    local -a indies=( ${!resolve_target_array[@]} )
+    local first_item=${indies[0]}
     if [ ! -z "$DEBUG_BUILDOR_GEN_DEPS" ]; then
-      echo ADDING a match: $src_key ${resolve_target_array[0]}
+      log_it ADDING a match: [$src_key]=$first_item
     fi
-    # for unique matches, we will store the correspondence so we can look
-    # it up very quickly later.
-    resolve_matches_src+=($src_key)
-    resolve_matches_dest+=(${resolve_target_array[0]})
+    # for unique matches, we will store the correspondence so we can find it very quickly later.
+    resolve_matches[$src_key]="$first_item"
+    return $RET_OKAY
   fi
+  return $RET_FAIL
 }
 #
 ############################################################################
 
-# main function that recurses on files and their dependencies.
-# this takes a list of file names to examine.  each one will have its
-# dependencies crawled.  we attempt to recurse on as few items as possible
+# main function that iterates on files and their dependencies.  this takes
+# the variable name of a list of file names to examine.  each one will have
+# its dependencies crawled.  we attempt to recurse on as few items as possible
 # by making sure we haven't already seen files or decided they're bad.
-function recurse_on_deps {
-  # snag arguments into a list of dependencies to crawl.
-  local -a active_deps=($*)
+function recurse_on_deps()
+{
+  # the name of the list of dependencies to crawl is passed to us, initially with just
+  # one item.  it's important to realize that we're getting the external variable's name
+  # and then accessing it locally via the alias "active_deps" below.  as we find new items
+  # for the list, we add them to it.  when an item has been totally processed, it's removed
+  # from the list.
+  local -n active_deps="$1"; shift
 
-  # pull off the first dependency so we can get all of its includes.
-  local first_element="${active_deps[0]}"
-  active_deps=(${active_deps[*]:1})
+  while [ ${#active_deps[@]} -ne 0 ]; do
+    # pull off the first dependency so we can get all of its includes.
+    local -a indies=( ${!active_deps[@]} )
+    local first_element=${indies[0]}
+    # chop the element we're working on out of the active list.
+    unset active_deps[$first_element]
+    # invoke our workhorse method on the item.
+    chew_on_one_dependency ${!active_deps} "$first_element"
+  done
+  return 0
+}
+
+# processes one file to locate all of its dependencies.
+# the external global list will be updated as this runs.
+function chew_on_one_dependency()
+{
+  local -n active_deps="$1"; shift
+  local first_element="$1"; shift
 
   # make the best guess we can at the real path.
-  resolve_filename $first_element
-  local to_examine="${resolve_target_array[0]}"
+  if ! resolve_filename $first_element; then
+    log_it "-- FAILED to resolve the filename for '$first_element'"
+    return $RET_FAIL
+  fi
+  local -a indies=( ${!resolve_target_array[@]} )
+  local to_examine=${indies[0]}
+  if [ ! -z "$DEBUG_BUILDOR_GEN_DEPS" ]; then
+    log_it "resolved path to_examine is '$to_examine'"
+  fi
 
-  # we didn't already have a failure (due to it being a bad file already
+  # we didn't already see a failure (due to it being a bad file already
   # or other problems).  and once we execute the below code to grab the
-  # file's dependencies, it really is boring and we never want to see it
-  # again.
-  boring_files+=($to_examine)
+  # file's dependencies, the file really will be boring and we never want
+  # to see it again in our processing.
+  boring_files[$to_examine]=yawn
 
   local dirtmp=$(dirname "$to_examine")
   local basetmp=$(basename "$to_examine")
-  echo "dependent on: $(basename "$dirtmp")/$basetmp"
+
+  log_it "++ dependent on: $(basename "$dirtmp")/$basetmp"
 #hmmm: do a better, nicer output--gather the dependencies listed in debugging
 #      line above into a list that will be printed out at the end.
 
@@ -264,11 +276,13 @@ function recurse_on_deps {
   local partial_file="$(mktemp $TEMPORARIES_PILE/zz_buildor_deps_filepart_${basetmp}.XXXXXX)"
   \rm -f "$partial_file"
 
+  if [ ! -z "$DEBUG_BUILDOR_GEN_DEPS" ]; then
+    log_it "\n\n========\nprocessing file '$to_examine'\n========"
+  fi
+
   # find all the includes in this file and save to the temp file.
   while read -r spoon; do
-    has_guard="$(echo "$spoon" \
-        | sed -n -e 's/#ifdef __BUILD_STATIC_APPLICATION__/yep/p')" 
-    if [ ! -z "$has_guard" ]; then
+    if [[ $spoon == *"#ifdef __BUILD_STATIC_APPLICATION__"* ]]; then
       # quit reading when we've seen the start of one of our guards.
       break
     fi
@@ -281,57 +295,56 @@ function recurse_on_deps {
   \rm "$partial_file"
 
   if [ ! -z "$DEBUG_BUILDOR_GEN_DEPS" ]; then
-    echo "grabbing includes from: $to_examine"
+    log_it "grabbing includes from: $to_examine"
   fi
 
 #hmmm: could separate the find deps on this file stuff below.
 
   local fp_dir=$(dirname "$to_examine")
-#echo fp_dir is: $fp_dir
 
-  # iterate across the dependencies we saw and add them to our list if
-  # we haven't already.
+  # iterate across the dependencies we saw and add them to our list if we haven't already.
   while read -r line_found; do
-    local chew_toy=$(echo $line_found | sed -e 's/^[ \t]*#include *<\(.*\)>.*$/\1/')
-    local original_value="$chew_toy"
-    # we want to add the file to the active list before we forgot about it.
+    # process the line to see if we can get a simple filename out of the include.
+    # we are only trying for system-searched files for this one, with angle brackets.
+    local chew_toy="${line_found#*\#include *<}"
+    chew_toy="${chew_toy/>*}"
     if [ ! -z "$DEBUG_BUILDOR_GEN_DEPS" ]; then
-      echo A: chew_toy=$chew_toy
+      log_it A: chew_toy=$chew_toy
     fi
 
     # check whether the dependency looks like one of our style of includes.
     # if it doesn't have a slash in it, then we need to give it the same
     # directory as the file we're working on.
-    local slash_present=$(echo $chew_toy | sed -n -e 's/.*[\\\/].*/yep/p')
+    local slash_present="${chew_toy/[^\/]*/}"
 
-    # the replacement above to get rid of #include failed.  try something
-    # simpler.
-    if [ ! -z "$(echo $chew_toy | sed -n -e 's/#include/crud/p')" ]; then
-      # try again with a simpler pattern.
-      chew_toy=$(echo $line_found | sed -e 's/^[ \t]*#include *[">]\(.*\)[">].*$/\1/') 
+    if [[ $chew_toy == *"#include"* ]]; then
+      # the replacement above to get rid of #include failed.  try something
+      # more inclusive, so we match double quote includes also.
+      chew_toy="${chew_toy#*\#include *[\"<]}"
+      chew_toy="${chew_toy/[\">]*}"
       if [ ! -z "$DEBUG_BUILDOR_GEN_DEPS" ]; then
-        echo B: chew_toy=$chew_toy
+        log_it B: chew_toy=$chew_toy
       fi
 
       # if it still has an #include or if it's not really a file, we can't
       # use it for anything.
-      if [ ! -z "$(echo $chew_toy | sed -n -e 's/#include/crud/p')" ]; then
-        echo "** bad include: $chew_toy"
+      if [[ $chew_toy == *"#include"* ]]; then
+        log_it "-- BAD include: $chew_toy"
         continue
       fi
 
-      # we are pretty sure that this file has no path components in it.
-      # we will add the surrounding directory if possible.
       if [ -z "$slash_present" ]; then
+        # we are pretty sure that this file has no path components in it.
+        # we will add the surrounding directory if possible.
         if [ -z "$fp_dir" ]; then
           # well, now we have no recourse, since we don't know where to
           # say this file comes from.
-          echo "** unknown directory: $chew_toy"
+          log_it "-- UNKNOWN directory: $chew_toy"
         else
           # cool, we can rely on the existing directory.
           chew_toy="$fp_dir/$chew_toy"
           if [ ! -z "$DEBUG_BUILDOR_GEN_DEPS" ]; then
-            echo patched dir: $chew_toy
+            log_it "patched dir: $chew_toy"
           fi
         fi
       fi
@@ -339,51 +352,41 @@ function recurse_on_deps {
 
     if bad_file $chew_toy; then
       if [ ! -z "$DEBUG_BUILDOR_GEN_DEPS" ]; then
-        echo "C: skipping because on bad list: $chew_toy"
+        log_it "C: skipping because on bad list: $chew_toy"
       fi
       continue
     fi
 
-###  # if we've seen it before, we bail.
-###  if seen_already "$to_examine"; then 
-###echo bailing since seen before: $to_examine
-###return 0;
-### fi
-
-  # now remember that we've seen this file.  we only remember it if
+    # now remember that we've seen this file.  we only remember it if
     # make sure we can see this file already, or we will need to seek it out.
     if [ ! -f "$chew_toy" ]; then
       # not an obvious filename yet.  try resolving it.
       resolve_filename $chew_toy
-      declare -a found_odd=(${resolve_target_array[*]})
-#echo found-list-is: ${found_odd[*]}
+      declare -a found_odd=(${!resolve_target_array[@]})
       local odd_len=${#found_odd[*]}
-#echo odd len is $odd_len
       if [ $odd_len -eq 0 ]; then
         # whoops.  we couldn't find it.  probably a system header, so toss it.
-        if [ ! -z "$DEBUG_BUILDOR_GEN_DEPS" ]; then
-          echo "** ignoring: $chew_toy"
-        fi
-        bad_files+=($chew_toy)
+        log_it "-- ignoring missing file: $chew_toy"
+        bad_files[$chew_toy]=yup
         chew_toy=""
       elif [ $odd_len -eq 1 ]; then
         # there's exactly one match, which is very good.
         chew_toy="${found_odd[0]}"
         if [ ! -z "$DEBUG_BUILDOR_GEN_DEPS" ]; then
-          echo "C: chew_toy=$chew_toy"
+          log_it "C: chew_toy=$chew_toy"
         fi
       else
         # this is really wrong.  there are multiple files with the same name?
         # that kind of things makes debugger tools angry or stupid.
-        echo "** non-unique name: $chew_toy"
-        bad_files+=($chew_toy)
+        log_it "-- NON-UNIQUE name: $chew_toy"
+        bad_files[$chew_toy]=urf
         chew_toy=""
       fi
     fi
 
     if [ ! -z "$chew_toy" -a ! -f "$chew_toy" ]; then
-      echo "** failed to compute a real path for: $chew_toy"
-      bad_files+=($chew_toy)
+      log_it "-- FAILED to compute a real path for: $chew_toy"
+      bad_files[$chew_toy]=meh
       chew_toy=""
       continue
     fi
@@ -395,33 +398,36 @@ function recurse_on_deps {
       # add the dependency we found.
       if add_new_dep "$chew_toy"; then
         # if that worked, it's not existing or bad so we want to keep it.
-        if ! already_listed "$chew_toy" ${active_deps[*]}; then
+#        if ! already_listed "$chew_toy" ${active_deps[*]}; then
+        if [ -z "${active_deps[$chew_toy]}" ]; then
           # track the file for its own merits also (to squeeze more includes).
-          active_deps+=($chew_toy)
+          active_deps[$chew_toy]=fiz
         fi
       fi
 
       # now compute the path as if it was the implementation file (x.cpp)
       # instead of being a header.  does that file exist?  if so, we'd like
       # its dependencies also.
-#slow and calls external app:    local cpp_toy=$(echo -n $chew_toy | sed -e 's/^\([^\.]*\)\.h$/\1.cpp/')
       local cpp_toy="${chew_toy%.h}.cpp"  # sweet and fast using just bash variable expansion.
       if [ ! -z "$DEBUG_BUILDOR_GEN_DEPS" ]; then
-        echo "cpp_toy is '$cpp_toy' as derived from chew_toy '$chew_toy'"
+        log_it "cpp_toy is '$cpp_toy' as derived from chew_toy '$chew_toy'"
       fi
 
       # there's no point in adding it if the name didn't change.
       if [ "$cpp_toy" != "$chew_toy" ]; then
         resolve_filename $cpp_toy
+        local -a indies=( ${!resolve_target_array[@]} )
+        local found_it="${indies[0]}"
 #hmmm: what if too many matches occur?
-        found_it="${resolve_target_array[0]}"
 
         # if the dependency actually exists, then we'll add it to our list.
         if [ ! -z "$found_it" ]; then
           if add_new_dep "$found_it"; then
             # that was a new dependency, so we'll continue examining it.
-            if ! already_listed "$found_it" ${active_deps[*]}; then
-              active_deps+=($found_it)
+#            if ! already_listed "$found_it" ${active_deps[*]}; then
+            if [ -z "${active_deps[$found_it]}" ]; then
+#              active_deps+=($found_it)
+              active_deps[$found_it]=pop
             fi
           fi
         fi
@@ -431,9 +437,12 @@ function recurse_on_deps {
 
   \rm -f "$current_includes"
 
-  # keep going on the list after our modifications.
-  if [ ${#active_deps[*]} -ne 0 ]; then recurse_on_deps ${active_deps[*]}; fi
-  return 0
+#  # keep going on the list after our modifications.
+#  if [ ${#active_deps[@]} -ne 0 ]; then
+#    recurse_on_deps ${active_deps[@]}
+#  fi
+
+  return $RET_OKAY
 }
 
 # this takes the dependency list and adds it to our current file.
@@ -443,9 +452,9 @@ function write_new_version {
   local opening_guard_line="\n#ifdef __BUILD_STATIC_APPLICATION__\n  // static dependencies found by buildor_gen_deps.sh:"
   local closing_guard_line="#endif // __BUILD_STATIC_APPLICATION__\n"
 
-  if [ ! -z "$DEBUG_BUILDOR_GEN_DEPS" ]; then
-    echo "would write deps to: $code_file"
-  fi
+#  if [ ! -z "$DEBUG_BUILDOR_GEN_DEPS" ]; then
+    log_it "++ writing new dependencies to: $code_file"
+#  fi
 
   local base="$(basename "$code_file")"
   local replacement_file="$(mktemp $TEMPORARIES_PILE/zz_buildor_deps_replacement_${base}.XXXXXX)"
@@ -457,10 +466,8 @@ function write_new_version {
   declare -a blanks=()
   # read in our existing file.
   while read -r orig_line; do
-#echo "read: '$orig_line'"
     # if it's the beginning of our static app section, stop reading.
-    if [ ! -z "$(echo $orig_line \
-        | sed -n -e 's/#ifdef __BUILD_STATIC_APPLICATION__/yep/p')" ]; then
+    if [[ $orig_line == *"#ifdef __BUILD_STATIC_APPLICATION__"* ]]; then
       break
     fi
     if [ -z "$orig_line" ]; then
@@ -484,36 +491,41 @@ function write_new_version {
   \rm -f "$pending_deps"
 
   # iterate across all the dependencies we found.
-  for line_please in ${dependency_accumulator[*]}; do
-    
+  for line_please in "${!dependency_accumulator[@]}"; do
     # throw out any items that are in the same directory we started in.
-    if [ "$prohibited_directory" == "$(dirname $line_please)" ]; then
+    local the_dir="$(dirname $line_please)"
+    local the_base="$(basename $line_please)"
+    if [ "$prohibited_directory" == "$the_dir" ]; then
       if [ ! -z "$DEBUG_BUILDOR_GEN_DEPS" ]; then
-        echo "skipping prohibited: $line_please"
+        log_it "skipping prohibited: $line_please"
       fi
       continue
     fi
 
     # strip the line down to just the filename and single directory component.
-    local chewed_line=$(echo $line_please | sed -e 's/.*[\\\/]\(.*\)[\\\/]\(.*\)$/\1\/\2/')
+#    local chewed_line=$(echo $line_please | sed -e 's/.*[\\\/]\(.*\)[\\\/]\(.*\)$/\1\/\2/')
+    local chewed_line="$(echo "$(basename "$the_dir")/$the_base" )"
+#log_it chewed_line became: $chewed_line
 
-    if [ ! -z "$(echo $chewed_line | sed -n -e 's/\.h$/yow/p')" ]; then
-#      if [ ! -z "$DEBUG_BUILDOR_GEN_DEPS" ]; then
-        echo "skipping header file: $chewed_line"
-#      fi
+    # see if this matches the header file ending; we don't want to add those to the cpp code.
+    if [[ "$chewed_line" =~ ^.*\.h$ ]]; then
+      if [ ! -z "$DEBUG_BUILDOR_GEN_DEPS" ]; then
+        log_it "skipping header file: $chewed_line"
+      fi
       continue
     fi
 
+    # this one seems like a good code file to add, so chuck it in there.
     local new_include="  #include <$chewed_line>"
     if [ ! -z "$DEBUG_BUILDOR_GEN_DEPS" ]; then
-      echo "adding '$new_include'"
+      log_it "adding '$new_include'"
     fi
     echo "$new_include" >>"$pending_deps"
   done
 
   # check that our dependencies file is not empty still.
   if [ ! -s "$pending_deps" ]; then
-    echo "
+    log_it "
 We encountered a problem during the generation of dependencies.
 The temporary output file:
   '${pending_deps}'
@@ -526,7 +538,7 @@ the current code file:
 or within this script itself:
   '$0'
 "
-    exit 1
+    exit $RET_FAIL
   fi
 
   sort "$pending_deps" >>"$replacement_file"
@@ -535,12 +547,12 @@ or within this script itself:
 
   echo -e "$closing_guard_line" >>"$replacement_file"
 
-#echo "about to move replacement, diffs:"
-#diff "$replacement_file" "$code_file"
-#echo "--------------"
-#echo full file:
-#cat "$replacement_file"
-#echo "--------------"
+  if [ ! -z "$DEBUG_BUILDOR_GEN_DEPS" ]; then
+    log_it "about to replace file.  here are the diffs:"
+    log_it "--------------"
+    diff "$replacement_file" "$code_file"
+    log_it "--------------"
+  fi
 
   \mv "$replacement_file" "$code_file"
   exit_on_error "replacing the original file with updated dependency version"
@@ -550,13 +562,27 @@ function find_dependencies {
   local code_file=$1
 
   # initialize our globals.
-  dependency_accumulator=()
-  boring_files=()
+  unset dependency_accumulator
+  declare -gA dependency_accumulator
+  unset boring_files
+  declare -gA boring_files
 
   # start recursing with the first dependency being the file itself.
-  recurse_on_deps $code_file
+  if [ ! -z "$DEBUG_BUILDOR_GEN_DEPS" ]; then
+    log_it "\n\n========\nstarting recursion on dependencies...\n========"
+  fi
+
+  # set up our indirectly referenced dictionary of dependencies.
+  unset global_active_dependencies
+  declare -gA global_active_dependencies
+  global_active_dependencies[$code_file]=go
+
+  recurse_on_deps global_active_dependencies
 
   # create the new version of the file.
+  if [ ! -z "$DEBUG_BUILDOR_GEN_DEPS" ]; then
+    log_it "\n\n========\nwriting new version of code file...\n========"
+  fi
   write_new_version "$code_file"
 }
 
@@ -568,43 +594,53 @@ for curr_parm in $*; do
   echo ""
 
   # resets the bad list in between sessions.
-  bad_files=() 
-#echo bad_files initial: ${bad_files[*]} 
+  unset bad_files
+  declare -gA bad_files
 
   if [ -f "$curr_parm" ]; then
-    echo "scanning file: $curr_parm"
+    log_it "scanning file: $curr_parm"
     prohibited_directory="$(dirname "$curr_parm")"
     # get the absolute path of the containing directory with our freaky pwd trick.
     prohibited_directory="$( \cd "$prohibited_directory" && \pwd )"
-echo "for file, containing dir absolute is now: $prohibited_directory"
+    if [ ! -z "$DEBUG_BUILDOR_GEN_DEPS" ]; then
+      log_it "for file, containing dir absolute is now: $prohibited_directory"
+    fi
     # fix our filename to be absolute.
     temp_absolute="$prohibited_directory/$(basename "$curr_parm")"
     curr_parm="$temp_absolute"
-echo "curr_parm file: $curr_parm"
+    if [ ! -z "$DEBUG_BUILDOR_GEN_DEPS" ]; then
+      log_it "curr_parm file: $curr_parm"
+    fi
     find_dependencies "$curr_parm"
   elif [ -d "$curr_parm" ]; then
-    echo "scanning folder: $curr_parm"
+    log_it "scanning folder: $curr_parm"
     prohibited_directory="$(dirname "$curr_parm")"
     # get absolute path of the containing directory.
     prohibited_directory="$( \cd "$prohibited_directory" && \pwd )"
-echo "for dir, containing dir absolute is now: $prohibited_directory"
+    if [ ! -z "$DEBUG_BUILDOR_GEN_DEPS" ]; then
+      log_it "for dir, containing dir absolute is now: $prohibited_directory"
+    fi
     # set the directory to that absolute path.
     curr_parm="$prohibited_directory"
-echo "curr_parm dir: $curr_parm"
+    if [ ! -z "$DEBUG_BUILDOR_GEN_DEPS" ]; then
+      log_it "curr_parm dir: $curr_parm"
+    fi
     local base="$(basename "$curr_parm")"
     outfile="$(mktemp $TEMPORARIES_PILE/zz_buildor_deps_outfile_${base}.XXXXXX)"
     find "$curr_parm" -iname "*.cpp" >"$outfile"
     while read -r line_found; do
       if [ $? != 0 ]; then break; fi
-echo "looking at file: $line_found"
+      if [ ! -z "$DEBUG_BUILDOR_GEN_DEPS" ]; then
+        log_it "looking at file: $line_found"
+      fi
       find_dependencies "$line_found"
     done <"$outfile"
     \rm -f "$outfile"
   else
-    echo "parameter is not a file or directory: $curr_parm"
+    log_it "-- parameter is not a file or directory: $curr_parm"
   fi
 
-  echo "ignored: " ${bad_files[*]}
+  log_it "++ ignored these files: " ${!bad_files[@]}
 
   echo ""
   echo ""
