@@ -234,7 +234,9 @@ function checkin_list()
   local list="$(uniquify $*)"
 
   # turn repo list back into an array.
-  eval "repository_list=( ${REPOSITORY_LIST[*]} )"
+#no, wtf???  repolist_combined="$(uniquify ${REPOSITORY_LIST_TO_COMMIT[*]} ${REPOSITORY_LIST_TO_PULL[*]})"
+#  eval "repository_list=( $repolist_combined )"
+  eval "repository_list=( ${REPOSITORY_LIST_TO_COMMIT[*]} )"
 
   local outer inner
 
@@ -490,7 +492,7 @@ function checkout_list()
   local list="$(uniquify $*)"
 
   # turn repo list back into an array.
-  eval "repository_list=( ${REPOSITORY_LIST[*]} )"
+  eval "repository_list=( ${REPOSITORY_LIST_TO_PULL[*]} )"
 
   local outer inner
 
@@ -514,7 +516,7 @@ function checkout_list()
 }
 
 # does a careful update on all the folders in the specified list;
-# it looks in the REPOSITORY_LIST for those names and updates them.
+# it looks in the REPOSITORY_LIST_TO_PULL for those names and updates them.
 # this is just like checkout_list, but it's for the puffing up action
 # we need to do on git.
 function puff_out_list()
@@ -523,7 +525,7 @@ function puff_out_list()
   local list="$(uniquify $*)"
 
   # turn repo list back into an array.
-  eval "repository_list=( ${REPOSITORY_LIST[*]} )"
+  eval "repository_list=( ${REPOSITORY_LIST_TO_PULL[*]} )"
 
   local outer inner
 
@@ -551,6 +553,8 @@ function puff_out_list()
 # that are located under the directory passed as the first parameter.
 # if this does not result in any directories being found, then a recursive
 # upwards search is done for git repos, which wants the .git directory.
+# note that this will skip links, unless the directory passed is itself a
+# link; it will be entered, but links underneath it will be ignored.
 function generate_rev_ctrl_filelist()
 {
   local dir="$1"; shift
@@ -559,14 +563,19 @@ function generate_rev_ctrl_filelist()
   local tempfile=$(mktemp /tmp/zz_checkins.XXXXXX)
   echo -n >$tempfile
   local additional_filter
-  find $dirhere -follow -maxdepth $MAX_DEPTH -type d -iname ".svn" -exec echo {}/.. ';' >>$tempfile 2>/dev/null
+  # we will still enter into dirhere if it's a link, but we will not follow into sub-links.
+  find $dirhere/. -maxdepth $MAX_DEPTH -type d -iname ".svn" -exec echo {}/.. ';' >>$tempfile 2>/dev/null
+#-follow 
 
 #hmmm: how to get the report of things ABOVE here, which we need.
 #  can we do an exec using the seek writable?
 
-  find $dirhere -follow -maxdepth $MAX_DEPTH -type d -iname ".git" -exec echo {}/.. ';' >>$tempfile 2>/dev/null
+  # we will still enter into dirhere if it's a link, but we will not follow into sub-links.
+  find $dirhere/. -maxdepth $MAX_DEPTH -type d -iname ".git" -exec echo {}/.. ';' >>$tempfile 2>/dev/null
+#-follow 
 
   # CVS is not well behaved like git and (now) svn, and we seldom use it anymore.
+
   popd &>/dev/null
 
   # see if they've warned us not to try checking in within vendor hierarchies.
